/*
 *  Copyright (C) 2021 Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdio.h>
#include <inttypes.h>
#include <kernel/dpl/ClockP.h>
#include <kernel/dpl/SemaphoreP.h>
#include <drivers/ipc_notify.h>
#include "ti_drivers_open_close.h"
#include "ti_board_open_close.h"



 void ipc_rpmsg_echo_main_core_start(void);
 void ipc_rpmsg_echo_main_core_receive(void);
 void ipc_rpmsg_echo_main_core_send(uint64_t GTCReadTime);

 void ipc_notify_msg_handler_main_core(uint32_t remoteCoreId, uint16_t localClientId, uint32_t msgValue, int32_t crcStatus, void *args);


/* number of iterations of message exchange to do */
uint32_t gMsgEchoCount = 100000u;


#if defined(SOC_AM64X)
/* main core that starts the message exchange */
uint32_t gMainCoreId = CSL_CORE_ID_R5FSS0_0;
/* remote cores that echo messages from main core, make sure to NOT list main core in this list */
uint32_t gRemoteCoreId[] = {
    CSL_CORE_ID_A53SS0_0,
    CSL_CORE_ID_MAX /* this value indicates the end of the array */
};
#endif


volatile uint32_t  Read = 0xA5A5;


void ipc_rpmsg_echo_main_core_start(void)
{
    uint32_t  status;
    /* register a handler to receive messages */
    status = IpcNotify_registerClient(4U, ipc_notify_msg_handler_main_core, NULL);

    /* wait for all cores to be ready */
    IpcNotify_syncAll(SystemP_WAIT_FOREVER);

}


void ipc_rpmsg_echo_main_core_send(uint64_t GTCReadTime)
{

        //uint32_t  i;
        uint32_t  msgValue;
        uint32_t  status;

       /* msgBuf[0U] = (uint8_t)  ( (GTCReadTime>>56U) & 0x00000000000000FF );
        msgBuf[1U] = (uint8_t) ( (GTCReadTime>>48U ) &  0x00000000000000FF );
        msgBuf[2U] = (uint8_t) ( (GTCReadTime>>40U) &  0x00000000000000FF ) ;
        msgBuf[3U] = (uint8_t) ( (GTCReadTime>>32U) &  0x00000000000000FF );

        msgBuf[4U] = (uint8_t) ( (GTCReadTime>>24U) &  0x00000000000000FF ) ;
        msgBuf[5U] = (uint8_t) ((GTCReadTime>>16U) &  0x00000000000000FF ) ;
        msgBuf[6U] = (uint8_t) ((GTCReadTime>>8U) &  0x00000000000000FF ) ;
        msgBuf[7U] = (uint8_t) ((GTCReadTime & 0x00000000000000FF));

        msgSize = MAX_MSG_SIZE;*/

        msgValue = (uint32_t) GTCReadTime;

        /* send message's to all participating core's, wait for message to be put in HW FIFO */
        status = IpcNotify_sendMsg(gRemoteCoreId[0], 4U, msgValue, 1);
        DebugP_assert(status==SystemP_SUCCESS);


}

void ipc_notify_msg_handler_main_core(uint32_t remoteCoreId, uint16_t localClientId, uint32_t msgValue, int32_t crcStatus, void *args)
{

    Read = msgValue;

}

